@@ -7,10 +7,7 @@ import pytest
 import soco.soap
 from soco.exceptions import MusicServiceException
 from soco.music_services.accounts import Account
-from soco.music_services.music_service import (
-    MusicService,
-    MusicServiceSoapClient,
-)
+from soco.music_services.music_service import MusicService
 
 
 # Typical account data from http://{Sonos-ip}:1400/status/accounts
